import { globalLookup, globalSetting, signalValues } from "./global";
import { domContainer, pluginRenderTimeGrid, pluginRenderValues, mountTree,
         genOnWheel, genKeyHandler, keyBindo } from './wave-view';
import { findCurrentSignalValue } from './utils';

let mainRenderEl = null;
const canvasMap = new Map();

/**
 * 
 * @returns {Element}
 */
function getMainRenderEl() {
    if (!mainRenderEl) {
        const el = document.querySelectorAll('.vcd-render-wrapper')[0];
        if (el) {
            mainRenderEl = el;
        }
    }
    return mainRenderEl;
}


function pluginLocalStore(desc, pstate /* , els */) {
    const pstateJsonString = JSON.stringify({
        // yOffset: pstate.yOffset,
        xOffset: pstate.xOffset,
        xScale: pstate.xScale
    });
    localStorage.setItem('dide', pstateJsonString);
}

/**
 * 
 * @param {HTMLElement} parentElement 
 */
function makeWaveView(parentElement) {
    if (!parentElement) {
        parentElement = getMainRenderEl();
    }
    const container = domContainer({
        elemento: mountTree.defaultElemento,
        layers: mountTree.defaultLayers,
        renderPlugins: [
            pluginRenderTimeGrid,
            pluginRenderValues,
            pluginLocalStore
        ]
    });

    parentElement.appendChild(container.pstate.container);
    container.start(globalLookup);

    globalLookup.updater = () => {
        console.log('updater');
    };

    container.elo.container.addEventListener('wheel', genOnWheel(parentElement, container.pstate, globalLookup, keyBindo));
}


/**
 * @param {{
* kind: 'var' | 'scope',
* type: 'wire' | 'reg' | 'module',
* name: string,
* size: number,
* link: string
* }} signal
*/
function toggleRender(signal) {
    if (globalLookup.currentWires.has(signal)) {
        globalLookup.currentWires.delete(signal);
        delete globalLookup.currentSignalValues[signal.link];
        globalLookup.render();
    } else {
        globalLookup.currentWires.add(signal);
        const signalItem = globalLookup.chango[signal.link];
        const { wave, kind } = signalItem;
        const time = globalLookup.currentTime;
        globalLookup.currentSignalValues[signal.link] = findCurrentSignalValue(kind, wave, time);
        globalLookup.render();
    }
}

export {
    getMainRenderEl,
    makeWaveView,
    toggleRender
}