'use strict';

const xOffsetUpdate = require('./x-offset-update.js');
const xScaleUpdate = require('./x-scale-update.js');

const yScroll = delta => (pstate, cm) => {
    const info = cm.getScrollInfo();
    cm.scrollTo(null, info.top + info.clientHeight * delta);
    return false;
};

const pluso = {
    desc: 'Zoom in time',
    fn: pstate => xScaleUpdate(pstate, 10 / 9 * pstate.xScale)
};

const minuso = {
    desc: 'Zoom out time',
    fn: pstate => xScaleUpdate(pstate, 9 / 10 * pstate.xScale)
};

const fullo = {
    desc: 'All of time',
    fn: pstate => xScaleUpdate(pstate, pstate.xScaleMin)
};

const scroll = {
    left: {
        desc: 'Scroll into the past',
        fn: pstate => xOffsetUpdate(pstate, pstate.xOffset + pstate.time)
    },
    right: {
        desc: 'Scroll into the future',
        fn: pstate => xOffsetUpdate(pstate, pstate.xOffset - pstate.time)
    },
    up: {
        desc: 'scroll up',
        fn: yScroll(-.1)
    },
    down: {
        desc: 'scroll down',
        fn: yScroll(.1)
    }
};

const editable = {
    desc: 'Toggle edit mode',
    fn: (pstate, cm) => {
        console.log('editable', pstate, cm);
    }
};

module.exports = {
    // Alt +  <,  >.    left / right
    'Alt+,': scroll.left,
    'Alt+.': scroll.right,

    // Alt +  [  ]      home / end
    'Alt+[': { desc: 'Jump to beginning of time', fn: pstate => xOffsetUpdate(pstate, pstate.sidebarWidth) }, // Home

    'Alt+]': { desc: 'Jump to end time', fn: pstate => xOffsetUpdate(pstate, pstate.width - pstate.xScale * pstate.time) }, // End

    // ALT +  -  +
    'Alt+=': pluso,  // '+': pluso,  '=': pluso,
    'Alt+-': minuso, // '-': minuso, '_': minuso,
    'Alt+0': fullo, // 'Shift+f': fullo, F: fullo, 'Shift+F': fullo,
    'Alt+/': editable,

    // wheel
    'icon:scrollLeft': scroll.left,
    'icon:scrollRight': scroll.right,
    'Shift+icon:scrollUp': scroll.left,
    'Shift+icon:scrollDown': scroll.right,
    'Ctrl+icon:scrollUp': pluso,
    'Ctrl+icon:scrollDown': minuso,

    // CAN'T DO: Alt + e, d, f, l

    // ArrowUp:    scroll.up,    'Shift+ArrowUp':    scroll.up,
    // ArrowDown:  scroll.down,  'Shift+ArrowDown':  scroll.down,

    // PageUp:     {desc: 'scroll page up',    fn: yScroll(-1)},
    // PageDown:   {desc: 'scroll page down',  fn: yScroll(1)},

    nop: { fn: () => false }
};
