'use strict';

const tt = require('onml/tt.js');
const genSVG = require('onml/gen-svg.js');
const stringify = require('onml/stringify.js');
// const getPco = require('./get-pco.js');
const water = require('./water.js');
const bracer = require('./bracer.js');
const vline = require('./vline.js');
const getX = require('./get-x.js');
const vlineStylo = require('./vline-stylo.js');
const getLabel = require('./get-label.js');

const defs = ['defs',
    ['linearGradient', { id: 'valid' },
        ['stop', { offset: '30%', 'stop-color': 'hsla(100, 100%, 100%,  0)' }],
        ['stop', { offset: '90%', 'stop-color': 'hsla(100, 100%, 100%, .3)' }]
    ],
    ['linearGradient', { id: 'valid&ready' },
        ['stop', { offset: '30%', 'stop-color': 'hsla(100, 100%, 50%,  0)' }],
        ['stop', { offset: '90%', 'stop-color': 'hsla(100, 100%, 50%, .5)' }]
    ],
    ['linearGradient', { id: 'valid&~ready' },
        ['stop', { offset: '30%', 'stop-color': 'hsla(50, 100%, 50%,  0)' }],
        ['stop', { offset: '90%', 'stop-color': 'hsla(50, 100%, 50%, .5)' }]
    ],
    ...Object.keys(vlineStylo).map(key => {
        const e = vlineStylo[key];
        return ['filter', { id: 'neonGlow-' + key, width: 7, x: -3 },
            ['feGaussianBlur', { stdDeviation: 3, in: 'SourceAlpha', result: 'ablur' }],
            ['feFlood', { 'flood-color': `hsl(${e.h},100%,${e.l}%)`, result: 'xf' }],
            ['feComposite', { in: 'xf', in2: 'ablur', operator: 'in' }]
        ];
    })
];

function* renderValues(desc, pstate) {
    const { width, height, sidebarWidth, yOffset, yStep, topBarHeight, botBarHeight } = pstate;

    const currentWires = desc.currentWires;
    const view = [];
    for (const signal of currentWires) {
        view.push({
            kind: signal.kind,
            name: signal.name,
            ref: signal.link
        });
    }

    const ilen = height / yStep;
    const iskip = yOffset / yStep;

    const ml = genSVG(width, height - topBarHeight - botBarHeight);

    let ifirst = 0;
    for (let i = 0; i < ilen; i += 1) {
        const lane = view[i];
        if (lane && (lane.name || lane.kind)) {
            if (i > iskip) {
                break;
            }
            ifirst = i;
        }
    }
    ml.push(defs);
    yield;

    const markers = ['g'];
    ml.push(markers);

    for (let i = 0; i < (iskip + ilen); i++) {
        const lane = view[i + (ifirst | 0)];

        if (lane && lane.kind === 'DIZ') {
            markers.push(['g', tt(0, Math.round((i - (iskip - ifirst) + 1.18) * yStep))].concat(water(lane, desc, pstate)));
        } else if (lane && lane.kind === 'brace') {
            markers.push(['g', tt(0, Math.round((i - (iskip - ifirst) + 1.18) * yStep))].concat(bracer(lane, desc, pstate)));
        } else if (lane && lane.ref) {
            const chango = desc.chango[lane.ref];
            if (chango && chango.kind === 'vec') {
                const mLane = ['g', tt(0, Math.round((i - (iskip - ifirst) + .8) * yStep))];
                const { wave } = chango;
                const jlen = wave.length;

                perLane: {
                    let [tPre, vPre, mPre] = wave[0];
                    let xPre = getX(pstate, tPre);
                    const labeler = getLabel(lane);
                    for (let j = 1; j <= jlen; j++) {
                        const mark = wave[j];

                        const [tCur, vCur, mCur] = (mark || [desc.time, 0, 0]);
                        const xCur = getX(pstate, tCur);
                        // console.log(mark, vPre, mPre, vCur, mCur);

                        if (vPre !== undefined || mPre !== undefined) {
                            if (xPre > width && xCur > width) { // both time stamps to the right
                                break perLane;
                            }

                            if (!((xPre < sidebarWidth) && (xCur < sidebarWidth))) { // both time stamps to the left
                                const xPreNorm = ((xPre > sidebarWidth) ? xPre : sidebarWidth) | 0;
                                const xCurNorm = ((xCur < width) ? xCur : width) | 0;
                                const w = xCurNorm - xPreNorm;
                                // 宽度太小不画了
                                if (w > 8) {
                                    const x = Math.round((xPreNorm + xCurNorm) / 2);
                                    mLane.push(labeler(vPre, mPre, x, w));
                                }
                            }
                        }
                        xPre = xCur;
                        vPre = vCur;
                        mPre = mCur;
                    }
                }
                ml.push(mLane);
            }
            yield;
        }
    }

    // console.log(view);
    for (let i = 0; i < view.length; i++) {
        const lane = view[i];
        if (lane && lane.vlines) {
            markers.push(...vline(lane, pstate, i));
        }
    }
    yield;
    return stringify(ml);
}

module.exports = renderValues;
/* eslint complexity: [1, 55] */
